﻿Type=StaticCode
Version=2.2
B4J=true
@EndOfDesignText@
'Static code module
Sub Process_Globals

	Private db	As SQL

	Private oldRecord	As Map
	Private newRecord	As Map

	Private c_aa_number					As String = "aa_number"
	Private c_last_name					As String = "last_name"
	Private c_first_name				As String = "first_name"
	Private c_rgb						As String = "rgb"
	Private c_club						As String = "club"
	Private c_date_of_birth				As String = "date_of_birth"
	Private c_gender					As String = "gender"
	Private c_bow_type					As String = "bow_type"
	Private c_archer_category			As String = "archer_category"
	Private c_default_division_row_id	As String = "default_division_row_id"
	Private c_default_class_row_id		As String = "default_class_row_id"

End Sub

Public Sub Initialise(pDb As SQL)

	db	= pDb

	oldRecord.Initialize
	newRecord.Initialize

End Sub

Public Sub FillArcherDefaultClass(pDB As SQL, pList As List)

	DBUtils.ExecuteList(pDB, "SELECT ARCHER_CATEGORY FROM CLASS", Null, 0, pList)

End Sub

Public Sub FillArcherDefaultDivision(pDB As SQL, pList As List)

	DBUtils.ExecuteList(pDB, "SELECT BOW_TYPE FROM DIVISION", Null, 0, pList)

End Sub

Public Sub Fetch(pRowId As Int)

	oldRecord = DBUtils.ExecuteMap(db, "SELECT A.ROW_ID, A.LAST_NAME, IFNULL(A.FIRST_NAME, '') AS 'FIRST_NAME', IFNULL(strftime('%d-%m-%Y', A.DATE_OF_BIRTH),'') AS 'DATE_OF_BIRTH', IFNULL(A.AA_NUMBER, '') AS 'AA_NUMBER', IFNULL(A.RGB, '') AS 'RGB', IFNULL(A.CLUB, '') AS 'CLUB', IFNULL(D.BOW_TYPE, '') AS 'BOW_TYPE', IFNULL(C.ARCHER_CATEGORY, '') AS 'ARCHER_CATEGORY', IFNULL(A.GENDER, '') AS 'GENDER' FROM ARCHER A LEFT JOIN DIVISION D ON A.DEFAULT_DIVISION_ROW_ID = D.ROW_ID LEFT JOIN CLASS C ON A.DEFAULT_CLASS_ROW_ID = C.ROW_ID WHERE A.ROW_ID = ?", Array As String(pRowId))
	newRecord = MapUtils.CloneMap(oldRecord)

End Sub

Public Sub UpdateArcher()

	Dim values	As Map
	Dim filter	As Map

	values.Initialize
	filter.Initialize
	If oldRecord.Get(c_aa_number) <> newRecord.Get(c_aa_number) Then
		values.Put(c_aa_number, newRecord.Get(c_aa_number))
	End If
	If oldRecord.Get(c_last_name) <> newRecord.Get(c_last_name) Then
		values.Put(c_last_name, newRecord.Get(c_last_name))
	End If
	If oldRecord.Get(c_first_name) <> newRecord.Get(c_first_name) Then
		values.Put(c_first_name, newRecord.Get(c_first_name))
	End If
	If oldRecord.Get(c_rgb) <> newRecord.Get(c_rgb) Then
		values.Put(c_rgb, newRecord.Get(c_rgb))
	End If
	If oldRecord.Get(c_club) <> newRecord.Get(c_club) Then
		values.Put(c_club, newRecord.Get(c_club))
	End If
	If oldRecord.Get(c_date_of_birth) <> newRecord.Get(c_date_of_birth) Then
		If newRecord.Get(c_date_of_birth) = "" Then
			values.Put(c_date_of_birth, Null)
		Else
			values.Put(c_date_of_birth, StringUtils.ReformatDate(newRecord.Get(c_date_of_birth), "dd-MM-yyyy", "yyyy-MM-dd"))
		End If
	End If
	If oldRecord.Get(c_gender) <> newRecord.Get(c_gender) Then
		values.Put(c_gender, newRecord.Get(c_gender))
	End If
	If oldRecord.Get(c_bow_type) <> newRecord.Get(c_bow_type) Then
		If newRecord.Get(c_bow_type) = "" Then
			values.Put(c_default_division_row_id, newRecord.Get(c_bow_type))
		Else
			Dim row_id As String
			row_id = db.ExecQuerySingleResult2("Select ROW_ID FROM DIVISION WHERE BOW_TYPE = ?", Array As String(newRecord.Get(c_bow_type)))
			values.Put(c_default_division_row_id, row_id)
		End If
	End If
	If oldRecord.Get(c_archer_category) <> newRecord.Get(c_archer_category) Then
		If newRecord.Get(c_archer_category) = "" Then
			values.Put(c_default_class_row_id, newRecord.Get(c_archer_category))
		Else
			Dim row_id As String
			row_id = db.ExecQuerySingleResult2("Select ROW_ID FROM CLASS WHERE ARCHER_CATEGORY = ?", Array As String(newRecord.Get(c_archer_category)))
			values.Put(c_default_class_row_id, row_id)
		End If
	End If
	filter.Put("row_id", newRecord.Get("row_id"))
	DBUtils.UpdateRecord2(db, "ARCHER", values, filter)

End Sub

Public Sub rowIdGet() As String

	Return newRecord.Get("row_id")

End Sub

Public Sub aaNumberGet() As String

	Return newRecord.Get(c_aa_number)

End Sub

Public Sub aaNumberSet(pValue As String)

	newRecord.Put(c_aa_number, pValue)

End Sub

Public Sub lastNameGet() As String

	Return newRecord.Get(c_last_name)

End Sub

Public Sub lastNameSet(pValue As String)

	newRecord.Put(c_last_name, pValue)

End Sub

Public Sub firstNameGet() As String

	Return newRecord.Get(c_first_name)

End Sub

Public Sub firstNameSet(pValue As String)

	newRecord.Put(c_first_name, pValue)

End Sub

Public Sub rgbGet() As String

	Return newRecord.Get(c_rgb)

End Sub

Public Sub rgbSet(pValue As String)

	newRecord.Put(c_rgb, pValue)

End Sub

Public Sub clubGet() As String

	Return newRecord.Get(c_club)

End Sub

Public Sub clubSet(pValue As String)

	newRecord.Put(c_club, pValue)

End Sub

Public Sub dateOfBirthGet(pFormat As String) As String

	If newRecord.Get(c_date_of_birth) = "" Then
		Return ""
	Else
		Dim ticks As Long
		DateTime.DateFormat = pFormat
		ticks = DateTime.DateParse(newRecord.Get(c_date_of_birth))
		Return DateTime.date(ticks)
	End If

End Sub

Public Sub dateOfBirthSet(pValue As String)

	newRecord.Put(c_date_of_birth, pValue)

End Sub

Public Sub genderGet() As String

	Return newRecord.Get(c_gender)

End Sub

Public Sub genderSet(pValue As String)

	newRecord.Put(c_gender, pValue)

End Sub

Public Sub bowTypeGet() As String

	Return newRecord.Get(c_bow_type)

End Sub

Public Sub bowTypeSet(pValue As String)

	newRecord.Put(c_bow_type, pValue)

End Sub

Public Sub archerCategoryGet() As String

	Return newRecord.Get(c_archer_category)

End Sub

Public Sub archerCategorySet(pValue As String)

	newRecord.Put(c_archer_category, pValue)

End Sub

Public Sub isDirty() As Boolean

	Return MapUtils.CompareMap(oldRecord, newRecord)

End Sub

Public Sub validated() As Map

	Dim result	As Map
	Dim text	As StringBuilder
	Dim valid	As Boolean

	result.Initialize
	text.Initialize
	valid = True
	If Not(IsNumber(newRecord.Get(c_aa_number))) Then
		valid = False
		text.Append("AA Number can only contain numbers." & CRLF)
	End If
	If newRecord.Get(c_last_name) = "" Then
		valid = False
		text.Append("Last Name must contain a value." & CRLF)
	End If
	If newRecord.Get(c_date_of_birth) <> "" Then
		DateTime.DateFormat = "dd-MM-yyyy"
		Try
			DateTime.DateParse(newRecord.Get(c_date_of_birth))
		Catch
			valid = False
			text.Append("Date of birth is not a valid date." & CRLF)
		End Try
	End If
	result.Put("valid", valid)
	result.Put("errors", text.ToString)
	Return result

End Sub