﻿Type=Class
Version=2.2
B4J=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private fx As JFX
	Private frm As Form
	Private cmbYears As ComboBox
	Private cmbMonths As ComboBox
	Private month, year As Int
	Private boxW, boxH As Float
	Private vCorrection As Float
	Private tempSelectedDay As Int
	Private dayOfWeekOffset As Int
	Private daysInMonth As Int
	Private DaysPane As AnchorPane
	Private cvs As Canvas
	Private cvsBackground As Canvas
	Private selectedDate As Long
	Private selectedYear, selectedMonth, selectedDay As Int
	Private tempSelectedColor As Paint = fx.Colors.Cyan
	Private selectedColor As Paint = fx.Colors.RGB(144, 176, 248)
	Private cancel As Boolean
	Private lastLeft, lastTop As Double
	Private cvsDays As Canvas

	Private DaysTitlesPane As AnchorPane
End Sub

Public Sub Initialize(Owner As Form)

	frm.Initialize("frm", 300, 340)
	frm.RootPane.LoadLayout("DatePicker")
	frm.SetOwner(Owner)
	frm.Resizable = False
	frm.SetFormStyle("UTILITY")
	month = DateTime.GetMonth(DateTime.Now)
	year = DateTime.GetYear(DateTime.Now)
	For y = year To (year - 100) Step -1
		cmbYears.Items.Add(y)
	Next
	For Each m As String In DateUtils.GetMonthsNames
		cmbMonths.Items.Add(m)
	Next
	setDate(DateTime.Now)
	cvs.Initialize("cvs")
	cvsBackground.Initialize("")
	DaysPane.AddNode(cvsBackground, 0, 0, DaysPane.PrefWidth, DaysPane.PrefHeight)
	DaysPane.AddNode(cvs, 0, 0, DaysPane.PrefWidth, DaysPane.PrefHeight)
	boxW = cvs.Width / 7
	boxH = cvs.Height / 6
	vCorrection = 5
	cvsDays.Initialize("")
	DaysTitlesPane.AddNode(cvsDays, 0, 0, DaysTitlesPane.PrefWidth, DaysTitlesPane.PrefHeight)
	Dim daysFont As Font = fx.CreateFont("", 14, True, False)
	Dim i As Int
	For Each d As String In DateUtils.GetDaysNames
		cvsDays.DrawText(d.SubString2(0, 2), (i + 0.5) * boxW, 20, daysFont, fx.Colors.Gray, "CENTER")
		i = i + 1
	Next
	cvs.MouseCursor = fx.Cursors.HAND
End Sub

'returns true if the user selected a date
Public Sub Show As Boolean
	DrawDays
	cancel = True
	If lastLeft <> 0 AND lastTop <> 0 Then
		frm.WindowLeft = lastLeft
		frm.WindowTop = lastTop
	End If
	frm.ShowAndWait
	lastLeft = frm.WindowLeft
	lastTop = frm.WindowTop
	Return Not(cancel)
End Sub

Private Sub DrawDays
	cvs.ClearRect(0, 0, cvs.Width, cvs.Height)
	cvsBackground.ClearRect(0, 0, cvsBackground.Width, cvsBackground.Height)
	Dim firstDay As Long = DateUtils.setDate(year, month, 1) - 1
	dayOfWeekOffset = DateTime.GetDayOfWeek(firstDay) Mod 7
	daysInMonth = DateUtils.NumberOfDaysInMonth(month, year)
	If year = selectedYear AND month = selectedMonth Then
		'draw the selected box
		DrawBox(cvs, selectedColor, (selectedDay - 1 + dayOfWeekOffset) Mod 7, _
			(selectedDay - 1 + dayOfWeekOffset) / 7)
	End If
	Dim daysFont As Font = fx.CreateFont("", 14, True, False)
	For day = 1 To daysInMonth
		Dim row As Int = (day - 1 + dayOfWeekOffset) / 7
		cvs.DrawText(day, (((dayOfWeekOffset + day - 1) Mod 7) + 0.5) * boxW, _
			(row + 0.5)* boxH + vCorrection, daysFont, fx.Colors.Black, "CENTER")
	Next
End Sub

Private Sub DrawBox(c As Canvas, clr As Paint, x As Int, y As Int)
	c.DrawRect(x * boxW, y * boxH, boxW, boxH, clr, True, 0)
End Sub

'Gets or sets the selected date
Public Sub getDate As Long
	Return selectedDate
End Sub

Public Sub setDate(date As Long)
	year = DateTime.GetYear(date)
	month = DateTime.GetMonth(date)
	SelectDay(DateTime.GetDayOfMonth(date))
	cmbYears.SelectedIndex = year - cmbYears.Items.Get(0)
	cmbMonths.SelectedIndex = month - 1
End Sub

Private Sub SelectDay(day As Int)
	selectedDate = DateUtils.setDate(year, month, day)
	selectedDay = day
	selectedMonth = month
	selectedYear = year
End Sub
Private Sub cvs_MouseMoved (EventData As MouseEvent)
	HandleMouse(EventData.X, EventData.Y, True)
End Sub

Private Sub cvs_MousePressed (EventData As MouseEvent)
	HandleMouse(EventData.X, EventData.Y, False)
End Sub

Private Sub HandleMouse(x As Double, y As Double, move As Boolean)
	Dim boxX = x / boxW, boxY =  y / boxH As Int
	Dim newSelectedDay As Int = boxY * 7 + boxX + 1 - dayOfWeekOffset
	Dim validDay As Boolean = newSelectedDay > 0 AND newSelectedDay <= daysInMonth
	If move Then
		If newSelectedDay = tempSelectedDay Then Return
		cvsBackground.ClearRect(0, 0, cvsBackground.Width, cvsBackground.Height)
		tempSelectedDay = newSelectedDay
		If validDay Then
			DrawBox(cvsBackground, tempSelectedColor, boxX, boxY)
		End If
	Else
		cvsBackground.ClearRect(0, 0, cvsBackground.Width, cvsBackground.Height)
		If validDay Then
			SelectDay(newSelectedDay)
			cancel = False
			frm.Close
		End If
	End If
End Sub

Private Sub cmbMonths_SelectedIndexChanged(Index As Int, Value As Object)
	month = Index + 1
	DrawDays
End Sub

Private Sub cmbYears_SelectedIndexChanged(Index As Int, Value As Object)
	year = Value
	DrawDays
End Sub

Private Sub btnToday_Action
	setDate(DateTime.Now)
	cancel = False
	frm.Close
End Sub
Private Sub btnCancel_Action
	frm.Close
End Sub